# frozen_string_literal: true

require 'msgpack'

module CommunityTheatre
  class DisposableSpooler
    def initialize(dest, page_size: 100)
      @page_size = page_size
      @destination = dest
    end

    def spool!(enumerable)
      enumerable.each_slice(@page_size).each_with_index do |page, index|
        file_for_page(index) do |file|
          packer = MessagePack::Packer.new(file)
          packer.write_array_header(page.size)

          page.each { |item| packer.write(item) }

          packer.flush
        end
      end
    end

    # implementation of "starting_at" is only to the page level
    # (see #pages)
    def unspool(starting_at = 0)
      Enumerator.new do |yielder|
        pages(starting_at).each do |page_no|
          file_for_page(page_no, 'r') do |file|
            page = MessagePack::Unpacker.new(file).unpack
            page.each { |item| yielder << item }
          end
        end
      end
    end

    private

    # implementation of "skip_items" is only to the page level
    def pages(skip_items = nil)
      skip_items ||= 0
      skip_pages = (skip_items / @page_size).floor

      existing_pages.drop(skip_pages)
    end

    def existing_pages
      Dir.entries(@destination)
         .map{ |n| File.join(@destination, n) }
         .filter { |n| File.file?(n) }
         .filter { |n| n =~ /\/\d+$/ }
         .map { |n| /\/(\d+)$/.match(n)[1].to_i }
         .sort
    end

    def path_for_page(n)
      File.join(@destination, n.to_s)
    end

    def file_for_page(n, mode='w', &block)
      File.open(path_for_page(n), mode, &block)
    end
  end
end
